#include <jni.h>
#include <assert.h>
#include <vector>

using namespace std;

#include "org_scavino_twain_JTwain.h"
#include "windows.h"
#include "twain.h"
#include "ijl.h"
#include "jtwain.h"


/////////////////////////////////////////////////////////////////////////

/*
 * Class:     org.scavino.twain.JTwain
 * Method:    isTwainAvailble
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_scavino_twain_JTwain_isTwainAvailble
  (JNIEnv * , jobject){
	  jboolean _val = (jboolean)TWAIN_IsAvailable();
    
	return _val;
}

/*
 * Class:     org_scavino_twain_JTwain
 * Method:    getAvailableSources
 * Signature: ()[Ljava/lang/String;
 */
jobjectArray JNICALL Java_org_scavino_twain_JTwain_getAvailableSources
(JNIEnv* environment, jobject){

	TW_UINT16	ret = TWRC_FAILURE;
    TW_IDENTITY tw;
	memset(&tw,0,sizeof(tw));

	vector<string> sources;

	TWAIN_OpenSourceManager(NULL);

	if (TWAIN_State() >= TWAIN_SM_OPEN) {
		// TWAIN_Mgr(..) returns a int not a TWRC_SUCCESS code
		ret = (TW_UINT16)TWAIN_Mgr(DG_CONTROL, DAT_IDENTITY, MSG_GETFIRST, (TW_MEMREF)&tw);
	}
	
	if (ret == TRUE){
		// Parse the Source into a Java String
		string _firstProduct = tw.ProductName;
		sources.push_back(_firstProduct);
		do {
			ret = (TW_UINT16)TWAIN_Mgr(DG_CONTROL,DAT_IDENTITY,MSG_GETNEXT,(TW_MEMREF)&tw);
			if (ret==TRUE) {
				string _product = tw.ProductName;
				sources.push_back(_product);
			}
		}
		while(ret==TRUE);
	}

	TWAIN_UnloadSourceManager();

	jstring utf_str;
	jclass clazz;

	// Instantiate the String class
	clazz = environment->FindClass("java/lang/String");
	jobjectArray jSourceStrings = NULL;
	size_t arraySize = sources.size();
	jSourceStrings = environment->NewObjectArray((static_cast<jsize>(arraySize)),clazz,NULL);

	for ( size_t i = 0 ; i < arraySize; i++ ) {
		string _src = sources[i];
		utf_str = environment->NewStringUTF(_src.c_str());
		environment->SetObjectArrayElement(jSourceStrings,(static_cast<jsize>(i)),utf_str);
	}

	sources.clear();

	return jSourceStrings;

}

/*
 * Class:     org_scavino_twain_JTwain
 * Method:    acquire
 * Signature: ()Ljava/lang/String;
 */
jstring JNICALL Java_org_scavino_twain_JTwain_acquire__
(JNIEnv * , jobject){
	
	HWND	myHwnd	= GetValidHwnd(NULL);

	HANDLE dib = TWAIN_AcquireNative(myHwnd, 0);

	char szTempName[_MAX_PATH]; 
	char tempPathName[_MAX_PATH];
	GetTempPath(_MAX_PATH,tempPathName);

	GetTempFileName(tempPathName, // dir. for temp. files 
					"TWN",         // temp. file name prefix 
					1,             // create unique name 
					szTempName);   // buffer for name 
	TWAIN_WriteNativeToFilename(dib,szTempName);
	return NULL;
}

/*
 * Class:     org_scavino_twain_JTwain
 * Method:    acquire
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
jstring JNICALL Java_org_scavino_twain_JTwain_acquire__Ljava_lang_String_2
(JNIEnv * env, jobject , jstring str){
	
	const char *	utf_string;
	jboolean		isCopy;

	utf_string = env->GetStringUTFChars(str,&isCopy);

	string _sourceName = utf_string;


    if (TWAIN_State() < TWAIN_SM_OPEN) {
        if (TWAIN_OpenSourceManager(NULL) == 0) {
			OutputDebugString("Unable to open Source Manager");
		}
	}
	
	
	int _result = TWAIN_OpenSource(_sourceName.c_str());
	// This could be another parameter in the API
	TWAIN_SetHideUI(FALSE);

	char szTempName[_MAX_PATH]; 
	char tempPathName[_MAX_PATH];
	int pathOk = GetTempPath(_MAX_PATH,tempPathName);

	/* I would use the above method but it can unreliable
	 *
	 * The GetTempPath function does not verify 
 	 * that the directory specified by the TMP 
	 * or TEMP environment variables exists. The 
	 * function gets the temporary file path as follows: 
	 *
	 * The path specified by the TMP environment variable. 
	 * The path specified by the TEMP environment variable, if TMP is not defined.
	 * The Windows directory, if both TMP and TEMP are not defined. 
	 */

	if (pathOk) {
		GetTempFileName(tempPathName, // dir. for temp. files 
					"TW_JPG",         // temp. file name prefix 
					1,             // create unique name 
					szTempName);   // buffer for name 
	} else {
		GetTempFileName("C:\\", // dir. for temp. files 
					"TW_JPG",         // temp. file name prefix 
					1,             // create unique name 
					szTempName);   // buffer for name 
	}

	HANDLE	_hdib	= NULL;
	BOOL	_jpegResult	= FALSE;
	jstring _jImgFileName = NULL;

	if (_result){
		_hdib = TWAIN_AcquireNative(NULL,TWAIN_ANYTYPE);
		if (_hdib != NULL){
			_jpegResult = EncodeJPGFileFromDIB(szTempName, _hdib);
			TWAIN_FreeNative(_hdib);
		}
		
	}
	 
	if (isCopy == JNI_TRUE){
		env->ReleaseStringUTFChars(str,utf_string);
	}
	
	if (_jpegResult) {
		_jImgFileName = env->NewStringUTF(szTempName);
	}

	return _jImgFileName;
}