package org.scavino.JTwacker;

import java.awt.image.BufferedImage;
import java.util.Hashtable;
import java.awt.Toolkit;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;

import javax.swing.filechooser.FileFilter;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JComboBox;
import javax.swing.JToolBar;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.SwingUtilities;

import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.sun.image.codec.jpeg.JPEGImageDecoder;


/** Unit Testing Class
 */
public class JTwacker extends JFrame {
    
    class JPEGPanel extends JPanel {
        
        /** Image for the inner class
         */        
        protected BufferedImage mJPEGPanelBufferedImage;
        
        /** Pnale to diaply the image
         */        
        public JPEGPanel() {
            // no op
        }
        
        /** Sets the bufferedimage into the class
         * @param bi BufferedImage
         */        
        public void setBufferedImage(BufferedImage bi) {
            if (bi == null) {
                return;
            }
            mJPEGPanelBufferedImage = bi;
            Dimension d = new Dimension(mJPEGPanelBufferedImage.getWidth(this),
            mJPEGPanelBufferedImage.getHeight(this));
            setPreferredSize(d);
            revalidate();
            repaint();
        }
        
        /** Paints the component.
         * @param g Graphics object used for the painting
         */        
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            Dimension d = getSize();
            g.setColor(getBackground());
            g.fillRect(0, 0, d.width, d.height);
            if (mJPEGPanelBufferedImage != null) {
                g.drawImage(mJPEGPanelBufferedImage, 0, 0, this);
            }
        }
    }
    
    /** The panel containing and displaying the JPEG file
     */    
    protected JPEGPanel mJpegPanel;
    
    /** The JPEG file as a buffered image
     */    
    protected BufferedImage mBufferedImage;
    /** Display the availble TWAIN sources (o..n)
     */    
    protected JComboBox     mSourcesCombo;
    /** Displays the acquire button.  The TWAIN folks would like it if you added a
     * select source option in your aplication but do allow applications to display
     * the sources in an alternative mode
     */    
    protected JToolBar      mToolBar;
    
    /** Constructor
     */    
    public JTwacker() {
        super("JTWacker Demo");
        
        mJpegPanel = new JPEGPanel();
        JScrollPane ps = new JScrollPane(mJpegPanel,
        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(ps, BorderLayout.CENTER);
        
        WindowListener wndCloser = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        };
        addWindowListener(wndCloser);
        
        mToolBar = new JToolBar("Twain");
        mToolBar.setFloatable(false);
        addButtons();
        getContentPane().add(mToolBar, BorderLayout.NORTH);
        setSize(800, 600);
        
        /* Center the frame */
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        Rectangle frameDim = getBounds();
        setLocation(
        (screenDim.width  - frameDim.width) / 2,
        (screenDim.height - frameDim.height) / 2
        );
        setVisible(true);
    }
    
    /** Add the buttons to the toolbar
     */    
    protected void addButtons(){
        
        JButton _ab = new JButton("Acquire");
        _ab.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                acquireImage();
            }
        });
        mToolBar.add(_ab);
        mToolBar.addSeparator();
        
        if (org.scavino.twain.JTwain.getInstance().isTwainAvailble()) {
            String[] twainSources = org.scavino.twain.JTwain.getInstance().getAvailableSources();
            if (twainSources != null) {
                mSourcesCombo = new JComboBox(twainSources);
            } else {
                mSourcesCombo = new JComboBox();
                mSourcesCombo.addItem("<NONE AVAILABLE>");
            }
        } else {
            mSourcesCombo = new JComboBox();
            mSourcesCombo.addItem("<NONE AVAILABLE>");
        }
        mToolBar.add(mSourcesCombo);
    }
    
    
    /** Acquire the actual image through the JTwain interface
     */    
    protected void acquireImage() {
        if (org.scavino.twain.JTwain.getInstance().isTwainAvailble()){
            if (mSourcesCombo.getItemCount() > 0 ){
                String _source = (String)mSourcesCombo.getSelectedItem();
                if (_source != null){
                    String _filename = org.scavino.twain.JTwain.getInstance().acquire(_source);
                    if (_filename != null && _filename.length() > 0) {
                        File fChoosen = new File(_filename);
                        openFile(fChoosen);
                    } else {
                        System.out.println("Severe Error File Did not Acquire");
                    }
                } // end if
            } // end if
        } // end if
    }
    
    
    /** Used to open the file and then display in the JPEG Panel
     * @param file The file object to display
     */    
    protected void openFile(final File file) {
        if (file == null || !file.exists()) {
            return;
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        
        Thread runner = new Thread() {
            public void run() {
                try {
                    FileInputStream in = new FileInputStream(file);
                    JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                    mBufferedImage = decoder.decodeAsBufferedImage();
                    in.close();
                    SwingUtilities.invokeLater( new Runnable() {
                        public void run() {
                            reset();
                        }
                    });
                }
                catch (Exception ex) {
                    ex.printStackTrace();
                }
                setCursor(Cursor.getPredefinedCursor(  Cursor.DEFAULT_CURSOR));
            }
        };
        runner.start();
    }
    
    /** Resets the Panel to a working restate.
     */    
    protected void reset() {
        if (mBufferedImage != null) {
            mJpegPanel.setBufferedImage(mBufferedImage);
        }
    }
    
    /** Unit Testing main
     * @param argv Arguments
     */    
    public static void main(String argv[]) {
        new JTwacker();
    }

}