package org.scavino.twain;

/**
 *
 * Freeware License - May 2002 - scavino.org
 *
 * Software distributed under the scavino.org freeware license is copyright 
 * by scavino.org. However, in the spirit of academic community, we do grant 
 * users of the software the right to share it freely with their colleagues, 
 * provided this is not done for commercial gain and that this banner is 
 * maintained in the source code. The software distributed in 
 * this way may represent work in progress, and bears no warranty, either 
 * expressed or implied by scavino.org, its employers, clients or any other 
 * representatives.
 *
 * Enquiries about commercial licensing of this software should be directed to 
 * our Director of Business Services Hugo Scavino  mailto:hscavino@scavino.org.
 *
 * JTwain.java
 *
 * For complete informaiton on the TWAIN please visit http://www.twaing.org/
 *
 * For freeware C/C++ implementation DataSource and DataManagers please visit 
 * http://www.dosadi.com/
 *
 * For a freeware C DLL JPEG implementation please see Intel JPEG Library
 * http://www.intel.com or 
 * http://www.intel.com/software/products/perflib/ijl/
 *
 * For more information on the UML and the Unified process please visit
 * http://www.rational.com/
 * http://www.omg.org/ or http://www.uml.org/
 *
 */

/**
 * Created on April 14, 2002, 10:14 AM  Minimal C JNI interface to the TWAIN
 * libray(DLL). Far more capabilities and other diagnosticc information can be
 * gathered and is available from the C TWAINAPI.  Alas this Java implementaion
 * only performs the minimum in order to acquire and load an image.
 * @version 2.1
 * @author scavino.org
 */
public class JTwain {

    /**
     * Staic instance used by singleton
     */
    private static final JTwain mInstance = new JTwain();

    /** Name of the C/C++ JNI DLL which communicates with the C TWAIN DLL.
     * The file name is case sensative and must NOT contain
     * the .DLL extension
     */
    protected final String DLL_NAME = "jtwain";
    
    
    /** Creates a new instance of JTwain
     * Once the source manager has been loaded, it is not normally unloaded
     * until this object is destroyed. This makes normal TWAIN operations
     * quite a bit faster.  Since the Java appliation will not release the
     * JNI DLL the TWAIN DLL is not released until the end of the JMM's lifetime
     * even though the scanning is complete Calling TwainAvailable the first time loads the source manager.
     */    
    private JTwain() {
        initLib();
    }
    
    /** Singleton interface point
     * Using the single pattern in order to prevent unessary DLL initialization
     * calls.  As only one device typically is active at a time this design
     * pattern is appropriate.
     * @return Static instance of class
     */    
    public static JTwain getInstance(){
        return mInstance;
    }

    /**
      * <CODE>true</CODE> if the TWAIN Datasource Manager is available and can
      * be loaded.  Does not actually load the devices's datasources.
      * But normally, the presence of the DSM means that at least one
      * datasource has been installed. IsAvailable is fast after the first call
      * It can be used to enable or disable menu items for example.
      * @return Determines if the underlying WIN32 OS has a TWAIN32 
      *         implementation installed.  Either a devive or the TWAIN Groups
      *          Sample SDK must be installed for this method to retuen 
      *         <CODE>true</CODE>
      */
    public native boolean isTwainAvailble();
    
    /** Returns the names of all the available TAWAIN devices.  These names are
     *  vendor specific and perhaps not localized.  The names are not
     *  guaranteed to be unique, and sometimes the names are not very
     *  descriptive in some cases.
     * @return String array of all the device names found in the OS
     */    
    public native String[] getAvailableSources();
    
    /** Opens and enables the default source. The string is not empty if
     * successful, empty if something goes wrong. If successful the filename
     * is returned.  Note if acquire is called again the previous image is lost
     * forever.  Clients are encouraged to rename the file or otherwise make a
     * local copy if the actual image must persist.
     * @return The fully qualified path and filename for the JPEG file created
     */
    public native String acquire();
    
    /** Opens and enables the named source. The string is not empty if
     * successful, empty if something goes wrong. If successful the filename
     * is returned.  Note if acquire is called again the previous image is lost
     * forever.  Clients are encouraged to rename the file or otherwise make a
     * local copy if the actual image must persist.
     * @return The fully qualified path and filename for the JPEG file created
     * @param sourceName The name of the source.  Preferably this name is a
     *      value derived from the <CODE>getAvailbleSources</CODE> method
     */
    public native String acquire(String sourceName);

    /*
     * Loads the C/JNI Libray
     */
    private void initLib(){

        try {
            
            System.loadLibrary(DLL_NAME);
        }
        
        finally {
            // Send to your logging subsystem
            System.out.println("Loading : " + DLL_NAME + ".dll");
        }
    }
}